/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.views.production;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.databinding.observable.map.WritableMap;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.core.databinding.observable.set.ISetChangeListener;
import org.eclipse.core.databinding.observable.set.SetChangeEvent;
import org.eclipse.core.databinding.util.Policy;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.patrikdufresne.managers.IManagerObserver;
import com.patrikdufresne.managers.ManagerEvent;
import com.patrikdufresne.managers.ManagerException;
import com.planimod.core.PlanimodManagers;
import com.planimod.core.Product;
import com.planimod.core.ProductEntry;
import com.planimod.core.ProductionEvent;
import com.planimod.core.Shift;

/**
 * This class provide an observable map between a shift object and a list of production events represented by a string.
 * i.e. This class is wrapping a Map&lt;Product, String&gt;
 * 
 * @author Patrik Dufresne
 * 
 */
public class ProductionEventLabelObservableMap extends WritableMap {

    /**
     * Private interface to avoid exposing observers implementation.
     * 
     * @author Patrik Dufresne
     * 
     */
    private class PrivateInterface implements IManagerObserver, ISetChangeListener {

        /**
         * Public constructor
         */
        public PrivateInterface() {
            // Nothing to do
        }

        @Override
        public void handleManagerEvent(ManagerEvent event) {
            getRealm().exec(new Runnable() {
                @Override
                public void run() {
                    doCompute();
                }
            });
        }

        @Override
        public void handleSetChange(SetChangeEvent event) {
            getRealm().exec(new Runnable() {
                @Override
                public void run() {
                    doCompute();
                }
            });
        }
    }

    public static String format(Product product, Integer count) {
        StringBuilder buf = new StringBuilder();
        format(buf, product, count);
        return buf.toString();
    }

    /**
     * Used to format the product name.
     * 
     * @param buf
     *            a string builder
     * @param product
     *            the product
     * @param count
     *            the number of position in the product.
     */
    public static void format(StringBuilder buf, Product product, Integer count) {
        String family = product.getFamily() == null || product.getFamily().isEmpty() ? null : product.getFamily();
        String name = product.getName() == null || product.getName().isEmpty() ? null : product.getName();
        String refId = product.getRefId() == null || product.getRefId().isEmpty() ? null : product.getRefId();
        if (family != null) {
            buf.append(family);
            if (refId != null) {
                buf.append(" - "); //$NON-NLS-1$
                buf.append(refId);
            }
        } else if (name != null) {
            buf.append(name);
            if (refId != null) {
                buf.append(" - "); //$NON-NLS-1$
                buf.append(refId);
            }
        } else if (refId != null) {
            buf.append(refId);
        } else {
            buf.append("unknown"); //$NON-NLS-1$
        }
        if (count != null) {
            buf.append(" ("); //$NON-NLS-1$
            buf.append(count);
            buf.append(")"); //$NON-NLS-1$
        }
    }

    private PlanimodManagers managers;

    private PrivateInterface privateInterface = new PrivateInterface();

    private IObservableSet events;

    /**
     * Create a new observable map to provide labels
     * 
     * @param events
     *            the list of production events
     * @param productionEventsMap
     *            an observable map between the production events and number of required position.
     */
    public ProductionEventLabelObservableMap(PlanimodManagers managers, IObservableSet events) {
        super(Shift.class, String.class);
        this.managers = managers;
        this.events = events;
        doCompute();
    }

    @Override
    public void clear() {
        throw new UnsupportedOperationException();
    }

    /**
     * Make this observable map dirty.
     */
    protected void doCompute() {
        // If the shifts list is empty, clean the map.
        if (this.events.size() == 0) {
            super.clear();
            return;
        }

        // Query the production events for known shifts.
        Map<ProductionEvent, ProductEntry> table;
        try {
            table = this.managers.getProductManager().listProductTableByProductionEvent(this.events);
        } catch (ManagerException e) {
            Policy.getLog().log(new Status(IStatus.ERROR, Policy.JFACE_DATABINDING, IStatus.OK, e.getMessage(), e));
            return;
        }
        Map<ProductionEvent, String> map = new HashMap<ProductionEvent, String>();
        for (Entry<ProductionEvent, ProductEntry> e : table.entrySet()) {
            map.put(e.getKey(), ProductionEventLabelObservableMap.format(e.getValue().getKey(), e.getValue().getValue()));
        }
        // Update the map with new labels.
        super.putAll(map);

    }

    @Override
    protected void firstListenerAdded() {
        super.firstListenerAdded();
        this.managers.addObserver(ManagerEvent.ALL, ProductionEvent.class, this.privateInterface);
        this.events.addSetChangeListener(this.privateInterface);
    }

    @Override
    protected void lastListenerRemoved() {
        super.lastListenerRemoved();
        this.managers.removeObserver(ManagerEvent.ALL, ProductionEvent.class, this.privateInterface);
        this.events.removeSetChangeListener(this.privateInterface);
    }

    @Override
    public Object put(Object key, Object value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void putAll(Map arg0) {
        throw new UnsupportedOperationException();
    }

    @Override
    public Object remove(Object key) {
        throw new UnsupportedOperationException();
    }

}
